﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Aspose.Tasks;
using Aspose.Tasks.Visualization;

namespace ImagePrototype
{
    /// <summary>
    /// Main application form.
    /// </summary>
    public partial class ImageRenderingForm : Form
    {
        /// <summary>
        /// Project to render.
        /// </summary>
        private Project project;

        /// <summary>
        /// List of all project's tasks. Just to cache them.
        /// </summary>
        private ArrayList tasks = new ArrayList();

        /// <summary>
        /// Public form constructor. You can set license here.
        /// </summary>
        public ImageRenderingForm()
        {
            InitializeComponent();
            //Set license here like below.
            //Aspose.Tasks.License license = new Aspose.Tasks.License();
            //license.SetLicense("Aspose.Tasks.lic");
        }

        /// <summary>
        /// Select project file to render.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btSelect_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "project files (*.mpp)|*.mpp";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                this.tbFile.Text = dlg.FileName;
                ProjectReader reader = new ProjectReader();
                using (FileStream fs = new FileStream(dlg.FileName, FileMode.Open, FileAccess.Read))
                {
                    project = reader.Read(fs);
                }
                LoadCheckList(project);
            }
        }

        /// <summary>
        /// Render project as an image.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btSave_Click(object sender, EventArgs e)
        {
            if (tbFile.Text != string.Empty && File.Exists(tbFile.Text))
            {
                try
                {
                    ArrayList uids = new ArrayList();
                    foreach (int index in clExpanded.CheckedIndices)
                        uids.Add(((Task)tasks[index]).Uid);
                    IRenderResult res = project.Export(RenderFormat.Image, PresentationFormat.GanttChart,
                        Path.Combine(Path.GetDirectoryName(this.GetType().Assembly.Location), "image.jpeg"),
                        new BaseImageRenderParam(null, Point.Empty, new Size((int)nuWidth.Value, (int)nuHeight.Value),
                        (int)nuZoom.Value, (int)nuPage.Value, cbLegend.Checked), this.cbExpandAll.Checked, uids);
                    if (res.Result != RenderResult.OK)
                        MessageBox.Show(res.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    else
                        MessageBox.Show("Image was saved into 'image.jpeg' file successfully.");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Load tasks to selection check list. There are parent tasks to be expanded.
        /// </summary>
        /// <param name="project">Project to load.</param>
        private void LoadCheckList(Project project)
        {
            this.clExpanded.Items.Clear();
            tasks = GetAllProjectTasks(project);
            foreach (Task task in tasks)
            {
                this.clExpanded.Items.Add(task.Id + ". " + task.Name);
            }
        }

        /// <summary>
        /// Auxiliary method to load all project's tasks.
        /// </summary>
        /// <param name="project">Project to load.</param>
        /// <returns>Array list of all project tasks.</returns>
        private ArrayList GetAllProjectTasks(Project project)
        {
            ArrayList res = new ArrayList();
            if(project.RootTask != null)
                foreach(Task task in project.RootTask.Children)
                {
                    res.Add(task);
                    res.AddRange(GetAllChildren(task));
                }
            return res;
        }

        /// <summary>
        /// Recursive method to load children.
        /// </summary>
        /// <param name="task">Task to load.</param>
        /// <returns>Array list of task's children.</returns>
        private ArrayList GetAllChildren(Task task)
        {
            ArrayList res = new ArrayList();
            if(task.Children != null)
                foreach (Task child in task.Children)
                {
                    res.Add(child);
                    res.AddRange(GetAllChildren(child));
                }
            return res;
        }
    }
}